@@ -8,6 +8,12 @@ has [qw(batch_size limit skip)] => 0;
 has [qw(collection hint id max_scan snapshot sort tailable)];
 has [qw(fields query)] => sub { {} };
 
+sub add_batch {
+  my ($self, $docs) = @_;
+  push @{$self->{results} ||= []}, @$docs;
+  return $self;
+}
+
 sub all {
   my ($self, $cb) = @_;
 
@@ -54,7 +60,7 @@ sub count {
   my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
 
   my $collection = $self->collection;
-  my $count      = bson_doc
+  my $command    = bson_doc
     count => $collection->name,
     query => $self->build_query,
     skip  => $self->skip,
@@ -62,14 +68,14 @@ sub count {
 
   # Non-blocking
   return $collection->db->command(
-    $count => sub {
+    $command => sub {
       my ($collection, $err, $doc) = @_;
       $self->$cb($err, $doc ? $doc->{n} : 0);
     }
   ) if $cb;
 
   # Blocking
-  my $doc = $collection->db->command($count);
+  my $doc = $collection->db->command($command);
   return $doc ? $doc->{n} : 0;
 }
 
@@ -78,17 +84,17 @@ sub distinct {
   my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
 
   my $collection = $self->collection;
-  my $distinct   = bson_doc
+  my $command    = bson_doc
     distinct => $collection->name,
     key      => $key,
     query    => $self->build_query;
 
   # Blocking
   my $db = $collection->db;
-  return $db->command($distinct)->{values} unless $cb;
+  return $db->command($command)->{values} unless $cb;
 
   # Non-blocking
-  $db->command($distinct => sub { shift; $self->$cb(shift, shift->{values}) });
+  $db->command($command => sub { shift; $self->$cb(shift, shift->{values}) });
 }
 
 sub explain {
@@ -110,7 +116,7 @@ sub next {
 sub rewind {
   my ($self, $cb) = @_;
 
-  delete $self->{$_} for qw(num results);
+  delete @$self{qw(num results)};
   return $cb ? $self->_defer($cb) : undef unless defined(my $id = $self->id);
   $self->id(undef);
 
@@ -168,8 +174,7 @@ sub _enough {
 sub _enqueue {
   my ($self, $reply) = @_;
   return unless $reply;
-  push @{$self->{results} ||= []}, @{$reply->{docs}};
-  return $self->id($reply->{cursor})->_dequeue;
+  return $self->add_batch($reply->{docs})->id($reply->{cursor})->_dequeue;
 }
 
 sub _finished {
@@ -318,6 +323,12 @@ Tailable cursor.
 L<Mango::Cursor> inherits all methods from L<Mojo::Base> and implements the
 following new ones.
 
+=head2 add_batch
+
+  $cursor = $cursor->add_batch($docs);
+
+Add batch of documents to cursor.
+
 =head2 all
 
   my $docs = $cursor->all;
